<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;

use App\Classes\FTN\{Message,Process};
use App\Models\{Echoarea,Echomail,Netmail,Setup,System};

class ProcessPacket implements ShouldQueue
{
	private const LOGKEY = 'PP-';

	use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

	private Message $msg;

	public function __construct(Message $msg)
	{
		// Some checks
		$this->msg = $msg;
	}

	/**
	 * When calling ProcessPacket - we assume that the packet is from a valid source
	 */
	public function handle()
	{
		// Load our details
		$ftns = Setup::findOrFail(config('app.id'))->system->addresses;

		// If we are a netmail
		if ($this->msg->isNetmail()) {
			// @todo Enable checks to reject old messages
			// @todo Enable checks to reject duplicate
			// @todo Enable checks to see if this is a file request or file send

			// Determine if the message is to this system, or in transit
			if ($ftns->search(function($item) { return $this->msg->tftn == $item->ftn; }) !== FALSE) {
				// @todo Check if it is a duplicate message
				// @todo Check if the message is from a system we know about

				$processed = FALSE;

				// If the message is to a bot, we'll process it
				foreach (config('process.robots') as $class) {
					if ($processed = $class::handle($this->msg)) {
						break;
					}
				}

				// We'll ignore messages from *fix users
				if (in_array(strtolower($this->msg->user_from),['filefix','areafix'])) {
					Log::info(sprintf('Ignoring Netmail to the Hub from (%s) [%s] - its from a bot.',$this->msg->user_from,$this->msg->fftn));

					$o = new Netmail;
					$o->to = $this->msg->user_to;
					$o->from = $this->msg->user_from;
					$o->subject = $this->msg->subject;
					$o->datetime = $this->msg->date;
					$o->tzoffset = $this->msg->date->utcOffset();

					$o->flags = $this->msg->flags;
					$o->cost = $this->msg->cost;
					$o->msgid = $this->msg->msgid;

					$o->fftn_id = ($x=$this->msg->fftn_o) ? $x->id : NULL;
					$o->tftn_id = ($x=$this->msg->tftn_o) ? $x->id : NULL;
					$o->msg = $this->msg->message_src;
					$o->local = TRUE;

					$o->save();

					$processed = TRUE;
				}

				// If not processed, no users here!
				if (! $processed) {
					$reject = [
						'ÚÄ¿ÚÄ¿  ÂÚÄ¿ÚÄ¿ÚÂ¿',
						'³  ³ÄÙ  ³³ÄÙ³   ³ ',
						'Á  ÀÄÙÀÄÙÀÄÙÀÄÙ Á '
					];

					Log::info(sprintf('Netmail to the Hub from (%s) [%s] but no users here.',$this->msg->user_from,$this->msg->fftn));

					$reply = "Your Netmail was not processed by the hub and cannot be delivered to anybody (as there are no users here).\r";

					$reply .= "\r";
					$reply .= "\r";
					$reply .= "This is your original message:\r";
					$reply .= "------------------------------ BEING MESSAGE ------------------------------\r";
					$reply .= sprintf("TO: %s\r",$this->msg->user_to);
					$reply .= sprintf("SUBJECT: %s\r",$this->msg->subject);
					$reply .= $this->msg->message;
					$reply .= "------------------------------ CONTROL LINES ------------------------------\r";
					$reply .= sprintf("DATE: %s\r",$this->msg->date->format('Y-m-d H:i:s'));
					$reply .= sprintf("MSGID: %s\r",$this->msg->msgid);

					foreach ($this->msg->kludge as $k=>$v)
						$reply .= sprintf("@%s: %s\n",strtoupper($k),$v);
					foreach ($this->msg->via as $via)
						$reply .= sprintf("VIA: %s\n",$via);

					$reply .= "------------------------------  END MESSAGE  ------------------------------\r";

					$o = new Netmail();
					$o->to = $this->msg->user_from;
					$o->from = Setup::PRODUCT_NAME;
					$o->subject = 'Message Undeliverable - '.$this->msg->msgid;
					$o->datetime = $this->msg->date;
					$o->tzoffset = $this->msg->date->utcOffset();

					$o->cost = 0;
					$o->flags = Message::FLAG_LOCAL;

					$o->fftn_id = ($x=$this->msg->tftn_o) ? $x->id : NULL;
					$o->tftn_id = ($x=$this->msg->fftn_o) ? $x->id : NULL;
					$o->reply = $this->msg->msgid;
					$o->msg = Process::format_msg($reply,$reject);

					$o->tagline = 'Do you think it was fate which brought us together? Nah, bad luck :(';
					$o->tearline = sprintf('%s (%04X)',Setup::PRODUCT_NAME,Setup::PRODUCT_ID);
					$o->save();
				}

			// If in transit, store for collection
			} else {
				Log::info(sprintf('Netmail [%s] in transit to (%s) [%s] from (%s) [%s].',
					$this->msg->msgid,
					$this->msg->user_to,$this->msg->tftn,
					$this->msg->user_from,$this->msg->fftn,
				));

				// @todo Check if the message is to a system we know about
				// @todo In transit loop checking
				// @todo In transit TRACE response

				$o = new Netmail;
				$o->to = $this->msg->user_to;
				$o->from = $this->msg->user_from;
				$o->subject = $this->msg->subject;
				$o->datetime = $this->msg->date;
				$o->tzoffset = $this->msg->date->utcOffset();

				$o->flags = $this->msg->flags;
				$o->cost = $this->msg->cost;
				$o->msgid = $this->msg->msgid;

				$o->fftn_id = ($x=$this->msg->fftn_o) ? $x->id : NULL;
				$o->tftn_id = ($x=$this->msg->tftn_o) ? $x->id : NULL;
				$o->msg = $this->msg->message_src;

				$o->save();
			}

		// Else we are echomail
		} else {
			$ea = Echoarea::where('name',$this->msg->echoarea)
				->where('domain_id',$this->msg->fboss_o->zone->domain_id)
				->single();

			// Check for duplicate messages
			if ($this->msg->msgid) {
				$o = Echomail::where('msgid',$this->msg->msgid)->single();

				if ($o) {
					Log::alert(sprintf('%s:! Ignoring duplicate echomail [%s] in [%s] from (%s) [%s] to (%s).',
						self::LOGKEY,
						$this->msg->msgid,
						$this->msg->echoarea,
						$this->msg->user_to,$this->msg->tftn,
						$this->msg->user_from,
					));

					return;
				}
			}

			// @todo Can the sender create it if it doesnt exist?
			// @todo Can the sender send messages to this area?
			// - Create it, or
			// - Else record in bad area

			// We know about this area, store it
			$o = new Echomail;
			$o->to = $this->msg->user_to;
			$o->from = $this->msg->user_from;
			$o->subject = $this->msg->subject;
			$o->datetime = $this->msg->date;
			$o->tzoffset = $this->msg->date->utcOffset();

			$o->fftn_id = ($x=$this->msg->fboss_o) ? $x->id : NULL;
			$o->echoarea_id = $ea?->id;
			$o->msgid = $this->msg->msgid;

			$o->msg = $this->msg->message_src;
			$o->path = $this->msg->pathaddress->pluck('id')->jsonSerialize();
			$o->rogue_path = $this->msg->rogue_path->jsonSerialize();
			$o->seenby = $this->msg->seenaddress->pluck('id')->jsonSerialize();
			$o->rogue_seen = $this->msg->rogue_path->jsonSerialize();
			$o->toexport = TRUE;

			$o->save();

			Log::info(sprintf('%s: - Echomail [%s] in [%s] from (%s) [%s] to (%s) - [%s].',
				self::LOGKEY,
				$this->msg->msgid,
				$this->msg->echoarea,
				$this->msg->user_to,$this->msg->tftn,
				$this->msg->user_from,
				$o->id,
			));

			// If the message is to a bot, we'll process it
			foreach (config('process.echomail') as $class) {
				if ($class::handle($this->msg)) {
					break;
				}
			}
		}
	}
}